% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "An Optimization View of MUSIC and Its Extension to Missing Data"
% arXiv:1806.03511, 2018
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figures 5 and 6.


%% Generate data matrix
clc; clear; close all;

f = [0.1 0.4 0.8];          % true frequencies
r = [0.92 0.98 0.85];       % true damping ratios
K = length(f);              % # of true frequencies, # of active eigenvalues of A(N x N)
N = 50;                     % # of sensors, system dimension
U = randn(N,K);             % eigenvectors of A
U = U*diag(1./norms(U));
V = pinv(U);
x0 = randn(N,1);            % initial state
c = V*x0;
c = c';                     % coefficients
M = 50;                     % # of time samples
Time = 0:M-1;
Time = Time';

v = [r rand(1,M-K)];
vand = fliplr(vander(v));
vand = vand';
r_vand = vand(:,1:K);
Exp = exp(1j*2*pi*Time*f);
Arf = r_vand.*Exp;
chat = c.*norms(Arf);
Arf = Arf*diag(1./norms(Arf));

Data = Arf*diag(chat)*U';       % true data matrix

Mp = 30;                        % number of meansurement per sensor
indx_me=randperm(N*M,Mp*N);     % observation index
CData=zeros(M,N);
CData(indx_me)=Data(indx_me);   % data matrix with random missing entries


%% Parameters estimation via MD-MUSIC
cvx_begin quiet
variable T(M,M) hermitian;
variable Xn(M,N) complex;
variable W(N,N) Hermitian;
dual variable Y0
minimize(0.5*trace(T)+0.5*trace(W))
subject to
Y0:[T   Xn; ...
    Xn'   W] == semidefinite(M+N,M+N);
Xn(indx_me) == CData(indx_me);
cvx_end
Q_NNM=-2*Y0(1:M,M+1:M+N);

ErrX_NNM = norm(Data-Xn,'fro')/norm(Data,'fro');

% Construct dual polynomial

fgrid = 0:0.01:0.99;
rgrid = 0:0.01:0.99;

Qnorm = zeros(length(fgrid),length(rgrid));
rhogrid1 = zeros(length(fgrid),length(rgrid));

for i = 1:length(rgrid)
    rg = rgrid(i)*ones(1,length(fgrid));
    if length(fgrid) < M
        v = [rg rand(1,M-length(fgrid))];
        vand = fliplr(vander(v));
        vand = vand';
        r_vand = vand(:,1:length(fgrid));
        Exp = exp(1j*2*pi*Time*fgrid);
        Agrid = r_vand.*Exp;
        Agrid = Agrid*diag(1./norms(Agrid));
        Qnorm(:,i) = norms(Q_NNM'*Agrid);
    else
        vand = fliplr(vander(rg));
        vand = vand';
        r_vand = vand(1:M,:);
        Exp = exp(1j*2*pi*Time*fgrid);
        Agrid = r_vand.*Exp;
        Agrid = Agrid*diag(1./norms(Agrid));
        Qnorm(:,i) = norms(Q_NNM'*Agrid);
    end
    rhogrid1(:,i) = rgrid(i)*exp(1j*2*pi*fgrid);
end

sval = 0.9999;
[idxf, idxr] = find(Qnorm>=sval);
idxrho = find(Qnorm>=sval);
Khat = length(idxr);
rhat = rgrid(idxr);
fhat = fgrid(idxf);

xx = real(exp(2*pi*1i*(0:0.001:1)));
yy = imag(exp(2*pi*1i*(0:0.001:1)));
zz = zeros(1,length(0:0.001:1));


%% Plot the true (r,f) pairs and the estimated (r,f) pairs
figure
lw = 2;
fs = 15;
ms = 5;
plot3(xx,yy,zz,'-k','LineWidth',lw);
grid on;
hold on;
mesh(real(rhogrid1),imag(rhogrid1),Qnorm);
y3 = plot3(r.*cos(2*pi.*f),r.*sin(2*pi.*f),ones(K,1),'ro','Linewidth',lw,'MarkerSize',ms+4);
y4 = stem3(rhat.*cos(2*pi.*fhat),rhat.*sin(2*pi.*fhat),Qnorm(idxrho),'b*','Linewidth',lw,'MarkerSize',ms-2);
xlabel('$r\cos(2\pi f)$','Interpreter','LaTex','FontSize',fs)
ylabel('$r\sin(2\pi f)$','Interpreter','LaTex','FontSize',fs)
zlabel('$||\mathcal{Q}(r,f)||_2$','Interpreter','LaTex','FontSize',fs);
h = legend([y3,y4], 'Ground Truth', 'MD-MUSIC');
set(gca,'FontSize',fs);
set(h,'FontSize',fs);

%% Plot data reconstruction error
figure
imagesc(abs(Data-Xn))
title('$|\mathbf{X}^\star-\widehat{\mathbf{X}}|$','Interpreter','LaTex','FontSize',fs)
set(gcf, 'Color', 'w');
set(gca,'FontSize',fs);
colorbar;

